﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace Benday.EditConnectionString
{
    class Program
    {
        static void Main(string[] args)
        {
            if (args == null || args.Length == 0)
            {
                DisplayUsage();
            }
            else
            {
                RunEditConnectionString(args);
            }
        }

        private static void DisplayUsage()
        {
            Console.WriteLine("Copyright 2016 -- Benjamin Day Consulting, Inc.");
            Console.WriteLine("http://www.benday.com");
            Console.WriteLine("@benday");
            Console.WriteLine();
            Console.WriteLine("EditConnectionString /filename:config_file_path /name:connection_name /value:new_connection_string_value");
        }
        private static void RunEditConnectionString(string[] args)
        {
            // /filename:
            // /value:
            // /name:

            var argsAsDictionary = GetArgsAsDictionary(args);

            // DebugArgDictionary(argsAsDictionary);

            if (ArgNameExists(argsAsDictionary, "filename") == true &
                ArgNameExists(argsAsDictionary, "value") == true &
                ArgNameExists(argsAsDictionary, "name") == true)
            {
                string filename = argsAsDictionary["filename"];
                string name = argsAsDictionary["name"];
                string value = argsAsDictionary["value"];

                Console.WriteLine("Preparing to update database connection string...");
                Console.WriteLine("Filename: '{0}'", filename);
                Console.WriteLine("Connection string name: '{0}'", name);
                Console.WriteLine("Value: '{0}'", value);
                
                DatabaseConfigurationFileEditor.SetConnectionString(
                    filename,
                    name,
                    value);

                Console.WriteLine("Connection string updated.");
            }
            else
            {
                DisplayUsage();
            }
        }

        private static bool ArgNameExists(Dictionary<string, string> args, string argName)
        {
            if (args.ContainsKey(argName) == false)
            {
                Console.Error.WriteLine("Argument '{0}' is missing.", argName);

                return false;
            }
            else
            {
                return true;
            }
        }

        private static void DebugArgDictionary(Dictionary<string, string> argsAsDictionary)
        {
            Console.WriteLine("Valid arg count: {0}", argsAsDictionary.Keys.Count);

            foreach (var key in argsAsDictionary.Keys)
            {
                Console.WriteLine("Arg: '{0}'; Value: '{1}';", key, argsAsDictionary[key]);
            }
        }
        
        private static Dictionary<string, string> GetArgsAsDictionary(string[] args)
        {
            var returnValue = new Dictionary<string, string>();

            foreach (var arg in args)
            {
                if (String.IsNullOrWhiteSpace(arg) == false && 
                    arg.StartsWith("/") == true &&
                    arg.Contains(":") == true) 
                {
                    CleanArgAndAddToDictionary(arg, returnValue);
                }
            }

            return returnValue;
        }
        private static void CleanArgAndAddToDictionary(string arg, Dictionary<string, string> args)
        {
            var argWithoutSlash = arg.Substring(1);

            int locationOfColon = argWithoutSlash.IndexOf(":");

            var argName = argWithoutSlash.Substring(0, locationOfColon);

            var argValue = argWithoutSlash.Substring(locationOfColon + 1).Trim();

            argValue = RemoveLeadingQuote(argValue);
            argValue = RemoveTrailingQuote(argValue);

            if (args.ContainsKey(argName) == false)
            {
                args.Add(argName, argValue);
            }
        }
        
        private static string RemoveLeadingQuote(string argValue)
        {
            if (argValue.StartsWith("\"") == true)
            {
                return argValue.Substring(1);
            }
            else
            {
                return argValue;
            }
        }

        private static string RemoveTrailingQuote(string argValue)
        {
            if (argValue.EndsWith("\"") == true)
            {
                return argValue.Substring(0, argValue.Length - 1);
            }
            else
            {
                return argValue;
            }
        }
    }
}